#!/usr/bin/perl -w
#########################################################################
#
# Serz Minus (Sergey Lepenkov), <abalama@cpan.org>
#
# Copyright (C) 1998-2023 D&D Corporation. All Rights Reserved
#
# This is free software; you can redistribute it and/or modify it
# under the same terms as Perl itself.
#
#########################################################################
use Test::More tests => 15;

use NoRFC::Number;

# V1 number (valid)
{
    my $num = "RFC-R00000000";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;

    ok($obj->is_valid, "V1 num is valid: $num") or diag(explain(\%d));
    #note(explain(\%d));
}

# V1 number (invalid)
{
    my $num = "RFC-R00000001";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;

    ok(!$obj->is_valid, "V1 num is invalid: $num") or diag(explain(\%d));
    #note(explain(\%d));
}

# Draft-R22089058
{
    my $num = "Draft-R22089058";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;

    ok($obj->is_valid, "V1 draft num: $num") or diag(explain(\%d));
    #note(explain(\%d));
}

# V2 with bar
{
    my $num = "NoRFC-R000000000";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;

    ok($obj->is_valid, "V2 with bar: $num") or diag(explain(\%d));
    #note(explain(\%d));
}

# V2 with space
{
    my $num = "NoRFC R2020162121";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;

    ok($obj->is_valid, "V2 with space: $num") or diag(explain(\%d));
    #note(explain(\%d));
}

# V2 with dots
{
    my $num = "R.2020.162.12.1";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;

    ok($obj->is_valid, "V2 with dots: $num") or diag(explain(\%d));
    #note(explain(\%d));
}

# V2 without dots
{
    my $num = "R2020162121";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;

    ok($obj->is_valid, "V2 without dots: $num") or diag(explain(\%d));
    #note(explain(\%d));
}

# V2 without status letter
{
    my $num = "2020162121";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;

    ok($obj->is_valid, "V2 without status letter: $num") or diag(explain(\%d));
    #note(explain(\%d));
}

# V2 too long number #1
{
    my $num = "2020.162.123456.8";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;

    ok($obj->is_valid, "V2 too long number 1: $num") or diag(explain(\%d));
    #note(explain(\%d));
}

# V2 too long number #2
{
    my $num = "20201621234568";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;

    ok($obj->is_valid, "V2 too long number 2: $num") or diag(explain(\%d));
    #note(explain(\%d));
}

# V2 eXtra draft
{
    my $num = "Draft X20201621234568";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;

    ok($obj->is_valid, "V2 eXtra draft: $num") or diag(explain(\%d));
    #note(explain(\%d));
}

# As string
{
    my $num = "2020.162.123456.8";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;
    is($obj->as_string, "NoRFC-R20201621234568", "As string") or diag(explain(\%d));
    #note(explain(\%d));
}

# As short string
{
    my $num = "2020.162.123456.8";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;
    is($obj->as_short, "20201621234568", "As short string") or diag(explain(\%d));
    #note(explain(\%d));
}

# As short dotted string
{
    my $num = "X2020.162.123456.8";
    my $obj = NoRFC::Number->new($num);
    my %d = $obj->as_hash;
    is($obj->as_dotted, "X.2020.162.123456.8", "As short dotted string") or diag(explain(\%d));
    #note(explain(\%d));
}

# Make the object
{
    $obj = NoRFC::Number->new->make(
        'type'    => 'Draft',
        'status'  => 'R',
        'year'    => 2021,
        'yday'    => 50,
        'nn'      => 1,
    );
    my %d = $obj->as_hash;
    is($obj->as_string, "Draft-R202105016", "Make the object") or diag(explain(\%d));
    #note(explain(\%d));
}

1;

__END__

prove -lv t/05-parse.t
